Translations = {};
Translations.en =
{
	addtags: "Add Tags",
	alreadyhaveacct: "Already a Pocket user?",
	continueff: "Continue with Firefox",
	errorgeneric: "There was an error when trying to save to Pocket.",
	learnmore: "Learn More",
	loginnow: "Log in",
	maxtaglength: "Tags are limited to 25 characters",
	mustbeconnected: "You must be connected to the Internet in order to save to Pocket. Please check your connection and try again.",
	onlylinkssaved: "Only links can be saved",
	pagenotsaved: "Page Not Saved",
	pageremoved: "Page Removed",
	pagesaved: "Saved to Pocket",
	processingremove: "Removing Page...",
	processingtags: "Adding tags...",
	removepage: "Remove Page",
	save: "Save",
	saving: "Saving...",
	signupemail: "Sign up with email",
	signuptosave: "Sign up for Pocket. It’s free.",
	suggestedtags: "Suggested Tags",
	tagline: "Save articles and videos from Firefox to view in Pocket on any device, any time.",
	taglinestory_one: "Click the Pocket Button to save any article, video or page from Firefox.",
	taglinestory_two: "View in Pocket on any device, any time.",
	tagssaved: "Tags Added",
	signinfirefox: "Sign in with Firefox",
  signupfirefox: "Sign up with Firefox",
	viewlist: "View List"
};

Translations.de =
{
  addtags: "Tags hinzufügen",
  alreadyhaveacct: "Sind Sie bereits Pocket-Nutzer?",
  continueff: "Mit Firefox fortfahren",
  errorgeneric: "Beim Speichern des Links bei Pocket ist ein Fehler aufgetreten.",
  learnmore: "Mehr erfahren",
  loginnow: "Anmelden",
  maxtaglength: "Tags dürfen höchsten 25 Zeichen lang sein.",
  mustbeconnected: "Bitte überprüfen Sie, ob Sie mit dem Internet verbunden sind.",
  onlylinkssaved: "Es können nur Links gespeichert werden",
  pagenotsaved: "Seite nicht gespeichert",
  pageremoved: "Seite entfernt",
  pagesaved: "Bei Pocket gespeichert",
  processingremove: "Seite wird entfernt…",
  processingtags: "Tags werden hinzugefügt…",
  removepage: "Seite entfernen",
  save: "Speichern",
  saving: "Speichern…",
  signupemail: "Mit E-Mail registrieren",
  signuptosave: "Registrieren Sie sich bei Pocket. Das ist kostenlos.",
  suggestedtags: "Vorgeschlagene Tags",
  tagline: "Speichern Sie Artikel und Videos aus Firefox bei Pocket, um sie jederzeit und auf jedem Gerät ansehen zu können.",
  taglinestory_one: "Klicken Sie auf die Pocket-Schaltfläche, um beliebige Artikel, Videos und Seiten aus Firefox zu speichern.",
  taglinestory_two: "Lesen Sie diese mit Pocket, jederzeit und auf jedem Gerät.",
  tagssaved: "Tags hinzugefügt",
  signinfirefox: "Mit Firefox anmelden",
  signupfirefox: "Mit Firefox registrieren",
  viewlist: "Liste anzeigen"
};

Translations.es =
{
  addtags: "Añadir etiquetas",
  alreadyhaveacct: "¿Ya tiene cuenta Pocket?",
  continueff: "Continuar con Firefox",
  errorgeneric: "Se ha producido un error al guardar el enlace en Pocket.",
  learnmore: "Saber más",
  loginnow: "Iniciar sesión",
  maxtaglength: "Las etiquetas están limitadas a 25 caracteres.",
  mustbeconnected: "Compruebe que tiene conexión a Internet.",
  onlylinkssaved: "Solo se pueden guardar enlaces",
  pagenotsaved: "Página no guardada",
  pageremoved: "Página eliminada",
  pagesaved: "Guardada en Pocket",
  processingremove: "Eliminando página…",
  processingtags: "Añadiendo etiquetas…",
  removepage: "Eliminar página",
  save: "Guardar",
  saving: "Guardando…",
  signupemail: "Regístrese con su correo.",
  signuptosave: "Regístrese en Pocket. Es gratis.",
  suggestedtags: "Etiquetas sugeridas",
  tagline: "Guarde artículos y vídeos desde Firefox en Pocket para verlos en cualquier dispositivo y en cualquier momento.",
  taglinestory_one: "Pulse el botón Pocket para guardar cualquier artículo, vídeo o página desde Firefox.",
  taglinestory_two: "Véalo en Pocket en cualquier dispositivo y en cualquier momento.",
  tagssaved: "Etiquetas añadidas",
  signinfirefox: "Inicie sesión con Firefox",
  signupfirefox: "Regístrese con Firefox",
  viewlist: "Ver lista"
};

Translations.ja =
{
  addtags: "タグを追加",
  alreadyhaveacct: "アカウントをお持ちですか？",
  continueff: "Firefox で続行",
  errorgeneric: "Pocket にリンクを保存中に問題が発生しました。",
  learnmore: "詳細",
  loginnow: "ログイン",
  maxtaglength: "タグは 25 文字までです。",
  mustbeconnected: "インターネットに接続されていることを確認してください。",
  onlylinkssaved: "リンクのみ保存できます",
  pagenotsaved: "ページを保存できませんでした",
  pageremoved: "ページを削除しました",
  pagesaved: "Pocket に保存しました",
  processingremove: "ページを削除中...",
  processingtags: "タグを追加中...",
  removepage: "ページを削除",
  save: "保存",
  saving: "保存中...",
  signupemail: "メールでアカウント登録",
  signuptosave: "Pocket にアカウント登録してください。無料です。",
  suggestedtags: "タグ候補",
  tagline: "Pocket でいつでもどこでも見られるよう、Firefox から記事や動画を保存できます。",
  taglinestory_one: "Firefox から記事や動画やページを保存するには、Pocket ボタンをクリックしてください。",
  taglinestory_two: "Pocket でいつでもどこでも見られます。",
  tagssaved: "タグを追加しました",
  signinfirefox: "Firefox でログイン",
  signupfirefox: "Firefox でアカウント登録",
  viewlist: "マイリストを表示"
};

Translations.ru =
{
  addtags: "Добавить теги",
  alreadyhaveacct: "Уже используете Pocket?",
  continueff: "Продолжить через Firefox",
  errorgeneric: "Не удалось сохранить в Pocket.",
  learnmore: "Узнайте больше",
  loginnow: "Войдите",
  maxtaglength: "Длина тега не должна превышать 25 символов.",
  mustbeconnected: "Убедитесь, что вы подключены к Интернет.",
  onlylinkssaved: "Можно сохранять только ссылки",
  pagenotsaved: "Страница не сохранена",
  pageremoved: "Страница удалена",
  pagesaved: "Сохранено в Pocket",
  processingremove: "Удаление страницы...",
  processingtags: "Добавление тегов...",
  removepage: "Удалить страницу",
  save: "Сохранить",
  saving: "Сохранение...",
  signupemail: "Регистрация по эл. почте",
  signuptosave: "Зарегистрируйтесь в Pocket. Это бесплатно.",
  suggestedtags: "Рекомендуемые теги",
  tagline: "Сохраняйте статьи и видео из Firefox для просмотра в Pocket на любом устройстве, в любой момент.",
  taglinestory_one: "Щёлкните по кнопке Pocket, чтобы сохранить любую статью, видео или страницу из Firefox.",
  taglinestory_two: "Просматривайте их в Pocket на любом устройстве, в любой момент.",
  tagssaved: "Теги добавлены",
  signinfirefox: "Войти через Firefox",
  signupfirefox: "Регистрация через Firefox",
  viewlist: "Просмотреть список"
};
