/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */


/* NOTE: THESE NEED TO STAY IN SYNC WITH LIGHT-THEME.CSS AND DARK-THEME.CSS.
   We are copy/pasting variables from light-theme and dark-theme,
   since they aren't loaded in this context (within commandlineoutput.xhtml
   and commandlinetooltip.xhtml). */
:root[devtoolstheme="light"] {
  --gcli-background-color: #ebeced; /* --theme-tab-toolbar-background */
  --gcli-input-focused-background: #f7f7f7; /* --theme-sidebar-background */
  --gcli-input-color: #18191a; /* --theme-body-color */
  --gcli-border-color: #aaaaaa; /* --theme-splitter-color */
}

:root[devtoolstheme="dark"] {
  --gcli-background-color: #343c45; /* --theme-toolbar-background */
  --gcli-input-focused-background: #252c33; /* --theme-tab-toolbar-background */
  --gcli-input-color: #b6babf; /* --theme-body-color-alt */
  --gcli-border-color: black; /* --theme-splitter-color */
}

.gcli-body {
  margin: 0;
  font: message-box;
  color: var(--gcli-input-color);
}

#gcli-output-root,
#gcli-tooltip-root {
  border: 1px solid var(--gcli-border-color);
  border-radius: 3px;
  background-color: var(--gcli-background-color);
}

#gcli-output-root {
  padding: 5px 10px;
  border-bottom-left-radius: 0;
  border-bottom-right-radius: 0;
  border-bottom: 0;
}

#gcli-tooltip-root {
  padding: 5px 0px;
}

#gcli-tooltip-connector {
  margin-top: -1px;
  margin-left: 8px;
  width: 20px;
  height: 10px;
  border-left: 1px solid var(--gcli-border-color);
  border-right: 1px solid var(--gcli-border-color);
  background-color: var(--gcli-background-color);
}

.gcli-tt-description,
.gcli-tt-error {
  padding: 0 10px;
}

.gcli-row-out {
  padding: 0 5px;
  line-height: 1.2em;
  border-top: none;
  border-bottom: none;
  color: var(--gcli-input-color);
}

.gcli-row-out p,
.gcli-row-out h1,
.gcli-row-out h2,
.gcli-row-out h3 {
  margin: 5px 0;
}

.gcli-row-out h1,
.gcli-row-out h2,
.gcli-row-out h3,
.gcli-row-out h4,
.gcli-row-out h5,
.gcli-row-out th,
.gcli-row-out strong,
.gcli-row-out pre {
  color: var(--gcli-input-color);
}

.gcli-row-out pre {
  font-size: 80%;
}

.gcli-row-out td {
  white-space: nowrap;
}

.gcli-out-shortcut,
.gcli-help-synopsis {
  padding: 0 3px;
  margin: 0 4px;
  font-weight: normal;
  font-size: 90%;
  border-radius: 3px;
  background-color: var(--gcli-background-color);
  border: 1px solid var(--gcli-border-color);
}

.gcli-out-shortcut:before,
.gcli-help-synopsis:before {
  color: var(--gcli-input-color);
  -moz-padding-end: 2px;
}

.gcli-help-arrow {
  color: #666;
}

.gcli-help-description {
  margin: 0 20px;
  padding: 0;
}

.gcli-help-parameter {
  margin: 0 30px;
  padding: 0;
}

.gcli-help-header {
  margin: 10px 0 6px;
}

.gcli-menu-name {
  -moz-padding-start: 8px;
}

.gcli-menu-desc {
  -moz-padding-end: 8px;
  color: var(--gcli-input-color);
}

.gcli-menu-name:hover,
.gcli-menu-desc:hover {
  background-color: var(--gcli-input-focused-background);
}

.gcli-menu-highlight,
.gcli-menu-highlight:hover {
  background-color: hsla(0,100%,100%,.1);
}

.gcli-menu-typed {
  color: hsl(25,78%,50%);
}

.gcli-menu-more {
  font-size: 80%;
  text-align: end;
  -moz-padding-end: 8px;
}

.gcli-addon-disabled {
  opacity: 0.6;
  text-decoration: line-through;
}

.gcli-breakpoint-label {
  font-weight: bold;
}

.gcli-breakpoint-lineText {
  font-family: monospace;
}
