%if 0
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
%endif

.popup-notification-icon {
  width: 64px;
  height: 64px;
  -moz-margin-end: 10px;
}

.popup-notification-icon[popupid="geolocation"] {
  list-style-image: url(chrome://browser/skin/Geolocation-64.png);
}

.popup-notification-icon[popupid="push"] {
  list-style-image: url(chrome://browser/skin/Push-64.png);
}

.popup-notification-icon[popupid="xpinstall-disabled"],
.popup-notification-icon[popupid="addon-progress"],
.popup-notification-icon[popupid="addon-install-blocked"],
.popup-notification-icon[popupid="addon-install-failed"],
.popup-notification-icon[popupid="addon-install-confirmation"],
.popup-notification-icon[popupid="addon-install-complete"] {
  list-style-image: url(chrome://mozapps/skin/extensions/extensionGeneric.png);
  width: 32px;
  height: 32px;
}

.popup-notification-icon[popupid="click-to-play-plugins"] {
  list-style-image: url(chrome://mozapps/skin/plugins/pluginBlocked-64.png);
}

.popup-notification-icon[popupid="web-notifications"] {
  list-style-image: url(chrome://browser/skin/notification-64.png);
}

.popup-notification-icon[popupid="indexedDB-permissions-prompt"],
.popup-notification-icon[popupid*="offline-app-requested"],
.popup-notification-icon[popupid="offline-app-usage"] {
  list-style-image: url(chrome://global/skin/icons/question-64.png);
}

.popup-notification-icon[popupid="password"] {
  list-style-image: url(chrome://mozapps/skin/passwordmgr/key-64.png);
}

.popup-notification-icon[popupid="webapps-install-progress"],
.popup-notification-icon[popupid="webapps-install"] {
  list-style-image: url(chrome://global/skin/icons/webapps-64.png);
}

.popup-notification-icon[popupid="bad-content"] {
  list-style-image: url(chrome://browser/skin/bad-content-blocked-64.png);
}

.popup-notification-icon[popupid="bad-content"][mixedblockdisabled],
.popup-notification-icon[popupid="bad-content"][trackingblockdisabled] {
  list-style-image: url(chrome://browser/skin/bad-content-unblocked-64.png);
}

.popup-notification-icon[popupid="webRTC-sharingDevices"],
.popup-notification-icon[popupid="webRTC-shareDevices"] {
  list-style-image: url(chrome://browser/skin/webRTC-shareDevice-64.png);
}

.popup-notification-icon[popupid="webRTC-sharingMicrophone"],
.popup-notification-icon[popupid="webRTC-shareMicrophone"] {
  list-style-image: url(chrome://browser/skin/webRTC-shareMicrophone-64.png);
}

.popup-notification-icon[popupid="webRTC-sharingScreen"],
.popup-notification-icon[popupid="webRTC-shareScreen"] {
  list-style-image: url(chrome://browser/skin/webRTC-shareScreen-64.png);
}

.popup-notification-icon[popupid="pointerLock"] {
  list-style-image: url(chrome://browser/skin/pointerLock-64.png);
}

/* Notification icon box */
#notification-popup-box {
  position: relative;
  background-color: #fff;
  background-clip: padding-box;
  padding-left: 3px;
  border-width: 0 8px 0 0;
  border-style: solid;
  border-image: url("chrome://browser/skin/urlbar-arrow.png") 0 8 0 0 fill;
  -moz-margin-end: -8px;
}

@conditionalForwardWithUrlbar@ > #forward-button[disabled] + #urlbar > #notification-popup-box {
  padding-left: 7px;
}

#notification-popup-box:-moz-locale-dir(rtl),
.notification-anchor-icon:-moz-locale-dir(rtl) {
  transform: scaleX(-1);
}

.notification-anchor-icon {
%ifdef MOZ_WIDGET_GTK
  list-style-image: url(moz-icon://stock/gtk-dialog-info?size=16);
%else
  list-style-image: url(chrome://global/skin/icons/information-16.png);
%endif
  width: 16px;
  height: 16px;
  margin: 0 2px;
}

.identity-notification-icon,
#identity-notification-icon {
  list-style-image: url(chrome://mozapps/skin/profile/profileicon.png);
  /* XXX: need HiDPI version */
}

.geo-notification-icon,
#geo-notification-icon {
  list-style-image: url(chrome://browser/skin/Geolocation-16.png);
}

#push-notification-icon {
  list-style-image: url(chrome://browser/skin/Push-16.png);
}

#addons-notification-icon {
  list-style-image: url(chrome://mozapps/skin/extensions/extensionGeneric-16.png);
}

.indexedDB-notification-icon,
#indexedDB-notification-icon {
  list-style-image: url(chrome://global/skin/icons/question-16.png);
}

#password-notification-icon {
  list-style-image: url(chrome://mozapps/skin/passwordmgr/key-16.png);
}

.webapps-notification-icon,
#webapps-notification-icon {
  list-style-image: url(chrome://global/skin/icons/webapps-16.png);
}

#plugins-notification-icon {
  list-style-image: url(chrome://browser/skin/notification-pluginNormal.png);
}

#plugins-notification-icon.plugin-hidden {
  list-style-image: url(chrome://browser/skin/notification-pluginAlert.png);
}

#plugins-notification-icon.plugin-blocked {
  list-style-image: url(chrome://browser/skin/notification-pluginBlocked.png);
}

#plugins-notification-icon {
  -moz-image-region: rect(0, 16px, 16px, 0);
}

#plugins-notification-icon:hover {
  -moz-image-region: rect(0, 32px, 16px, 16px);
}

#plugins-notification-icon:active {
  -moz-image-region: rect(0, 48px, 16px, 32px);
}

#notification-popup-box[hidden] {
  /* Override display:none to make the pluginBlockedNotification animation work
     when showing the notification repeatedly. */
  display: -moz-box;
  visibility: collapse;
}

#plugins-notification-icon.plugin-blocked[showing] {
  animation: pluginBlockedNotification 500ms ease 0s 5 alternate both;
}

@keyframes pluginBlockedNotification {
  from {
    opacity: 0;
  }
  to {
    opacity: 1;
  }
}

.bad-content-blocked-notification-icon,
#bad-content-blocked-notification-icon {
  list-style-image: url(chrome://browser/skin/bad-content-blocked-16.png);
}

.bad-content-unblocked-notification-icon,
#bad-content-unblocked-notification-icon {
  list-style-image: url(chrome://browser/skin/bad-content-unblocked-16.png);
}

.webRTC-shareDevices-notification-icon,
#webRTC-shareDevices-notification-icon {
  list-style-image: url(chrome://browser/skin/webRTC-shareDevice-16.png);
}

.webRTC-sharingDevices-notification-icon,
#webRTC-sharingDevices-notification-icon {
  list-style-image: url(chrome://browser/skin/webRTC-sharingDevice-16.png);
}

.webRTC-shareMicrophone-notification-icon,
#webRTC-shareMicrophone-notification-icon {
  list-style-image: url(chrome://browser/skin/webRTC-shareMicrophone-16.png);
}

.webRTC-sharingMicrophone-notification-icon,
#webRTC-sharingMicrophone-notification-icon {
  list-style-image: url(chrome://browser/skin/webRTC-sharingMicrophone-16.png);
}

.webRTC-shareScreen-notification-icon,
#webRTC-shareScreen-notification-icon {
  list-style-image: url(chrome://browser/skin/webRTC-shareScreen-16.png);
}

.webRTC-sharingScreen-notification-icon,
#webRTC-sharingScreen-notification-icon {
  list-style-image: url(chrome://browser/skin/webRTC-sharingScreen-16.png);
}

.web-notifications-notification-icon,
#web-notifications-notification-icon {
  list-style-image: url(chrome://browser/skin/notification-16.png);
}

.pointerLock-notification-icon,
#pointerLock-notification-icon {
  list-style-image: url(chrome://browser/skin/pointerLock-16.png);
}

.translate-notification-icon,
#translate-notification-icon {
  list-style-image: url(chrome://browser/skin/translation-16.png);
  -moz-image-region: rect(0px, 16px, 16px, 0px);
}

.translated-notification-icon,
#translated-notification-icon {
  list-style-image: url(chrome://browser/skin/translation-16.png);
  -moz-image-region: rect(0px, 32px, 16px, 16px);
}

.popup-notification-icon[popupid="servicesInstall"] {
  list-style-image: url(chrome://browser/skin/social/services-64.png);
}
#servicesInstall-notification-icon {
  list-style-image: url(chrome://browser/skin/social/services-16.png);
}

/* EME notifications */

.popup-notification-icon[popupid="drmContentPlaying"],
#eme-notification-icon {
  list-style-image: url("chrome://browser/skin/drm-icon.svg#chains");
}

#eme-notification-icon:hover:active {
  list-style-image: url("chrome://browser/skin/drm-icon.svg#chains-pressed");
}

#eme-notification-icon[firstplay=true] {
  animation: emeTeachingMoment 0.2s linear 0s 5 normal;
}

@keyframes emeTeachingMoment {
  0% {transform: translateX(0); }
  25% {transform: translateX(3px) }
  75% {transform: translateX(-3px) }
  100% { transform: translateX(0); }
}

/* HiDPI notification icons */
@media (min-resolution: 1.1dppx) {
  #notification-popup-box {
    border-image: url("chrome://browser/skin/urlbar-arrow@2x.png") 0 16 0 0 fill;
  }

  .notification-anchor-icon {
%ifdef MOZ_WIDGET_GTK
    list-style-image: url(moz-icon://stock/gtk-dialog-info?size=dialog);
%else
    list-style-image: url(chrome://global/skin/icons/information-32.png);
%endif
  }

  .webRTC-shareDevices-notification-icon,
  #webRTC-shareDevices-notification-icon {
    list-style-image: url(chrome://browser/skin/webRTC-shareDevice-16@2x.png);
  }

  .webRTC-sharingDevices-notification-icon,
  #webRTC-sharingDevices-notification-icon {
    list-style-image: url(chrome://browser/skin/webRTC-sharingDevice-16@2x.png);
  }

  .webRTC-shareMicrophone-notification-icon,
  #webRTC-shareMicrophone-notification-icon {
    list-style-image: url(chrome://browser/skin/webRTC-shareMicrophone-16@2x.png);
  }

  .webRTC-sharingMicrophone-notification-icon,
  #webRTC-sharingMicrophone-notification-icon {
    list-style-image: url(chrome://browser/skin/webRTC-sharingMicrophone-16@2x.png);
  }

  .webRTC-shareScreen-notification-icon,
  #webRTC-shareScreen-notification-icon {
    list-style-image: url(chrome://browser/skin/webRTC-shareScreen-16@2x.png);
  }

  .webRTC-sharingScreen-notification-icon,
  #webRTC-sharingScreen-notification-icon {
    list-style-image: url(chrome://browser/skin/webRTC-sharingScreen-16@2x.png);
  }

  .popup-notification-icon[popupid="webRTC-sharingDevices"],
  .popup-notification-icon[popupid="webRTC-shareDevices"] {
    list-style-image: url(chrome://browser/skin/webRTC-shareDevice-64@2x.png);
  }

  .popup-notification-icon[popupid="webRTC-sharingMicrophone"],
  .popup-notification-icon[popupid="webRTC-shareMicrophone"] {
    list-style-image: url(chrome://browser/skin/webRTC-shareMicrophone-64@2x.png);
  }

  .popup-notification-icon[popupid="webRTC-sharingScreen"],
  .popup-notification-icon[popupid="webRTC-shareScreen"] {
    list-style-image: url(chrome://browser/skin/webRTC-shareScreen-64@2x.png);
  }

%ifdef XP_MACOSX
/* OSX only until we have icons for Windows and Linux */
  .geo-notification-icon,
  #geo-notification-icon {
    list-style-image: url(chrome://browser/skin/Geolocation-16@2x.png);
  }

  #push-notification-icon {
    list-style-image: url(chrome://browser/skin/Push-16@2x.png);
  }

  .indexedDB-notification-icon,
  #indexedDB-notification-icon {
    list-style-image: url(chrome://global/skin/icons/question-32.png);
  }

  #addons-notification-icon {
    list-style-image: url(chrome://mozapps/skin/extensions/extensionGeneric.png);
  }

  #password-notification-icon {
    list-style-image: url(chrome://mozapps/skin/passwordmgr/key-16@2x.png);
  }

  .webapps-notification-icon,
  #webapps-notification-icon {
    list-style-image: url(chrome://global/skin/icons/webapps-16@2x.png);
  }

  #plugins-notification-icon {
    list-style-image: url(chrome://browser/skin/notification-pluginNormal@2x.png);
  }

  #plugins-notification-icon.plugin-hidden {
    list-style-image: url(chrome://browser/skin/notification-pluginAlert@2x.png);
  }

  #plugins-notification-icon.plugin-blocked {
    list-style-image: url(chrome://browser/skin/notification-pluginBlocked@2x.png);
  }

  #plugins-notification-icon {
    -moz-image-region: rect(0, 32px, 32px, 0);
  }

  #plugins-notification-icon:hover {
    -moz-image-region: rect(0, 64px, 32px, 32px);
  }

  #plugins-notification-icon:active {
    -moz-image-region: rect(0, 96px, 32px, 64px);
  }

  #bad-content-blocked-notification-icon {
    list-style-image: url(chrome://browser/skin/bad-content-blocked-16@2x.png);
  }

  #bad-content-unblocked-notification-icon {
    list-style-image: url(chrome://browser/skin/bad-content-unblocked-16@2x.png);
  }

  .web-notifications-notification-icon,
  #web-notifications-notification-icon {
    list-style-image: url(chrome://browser/skin/notification-16@2x.png);
  }

  .pointerLock-notification-icon,
  #pointerLock-notification-icon {
    list-style-image: url(chrome://browser/skin/pointerLock-16@2x.png);
  }

  .translate-notification-icon,
  #translate-notification-icon {
    list-style-image: url(chrome://browser/skin/translation-16@2x.png);
    -moz-image-region: rect(0px, 32px, 32px, 0px);
  }

  .translated-notification-icon,
  #translated-notification-icon {
    list-style-image: url(chrome://browser/skin/translation-16@2x.png);
    -moz-image-region: rect(0px, 64px, 32px, 32px);
  }

  .popup-notification-icon[popupid="geolocation"] {
    list-style-image: url(chrome://browser/skin/Geolocation-64@2x.png);
  }

  .popup-notification-icon[popupid="push"] {
    list-style-image: url(chrome://browser/skin/Push-64@2x.png);
  }

  .popup-notification-icon[popupid="web-notifications"] {
    list-style-image: url(chrome://browser/skin/notification-64@2x.png);
  }

  .popup-notification-icon[popupid="bad-content"] {
    list-style-image: url(chrome://browser/skin/bad-content-blocked-64@2x.png);
  }

  .popup-notification-icon[popupid="bad-content"][mixedblockdisabled],
  .popup-notification-icon[popupid="bad-content"][trackingblockdisabled] {
    list-style-image: url(chrome://browser/skin/bad-content-unblocked-64@2x.png);
  }

  .popup-notification-icon[popupid="pointerLock"] {
    list-style-image: url(chrome://browser/skin/pointerLock-64@2x.png);
  }

  .popup-notification-icon[popupid="servicesInstall"] {
    list-style-image: url(chrome://browser/skin/social/services-64@2x.png);
  }

  #servicesInstall-notification-icon {
    list-style-image: url(chrome://browser/skin/social/services-16@2x.png);
  }
%endif
}
