/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

%include ../../shared/devtools/widgets.inc.css

.side-menu-widget-group-checkbox .checkbox-spacer-box,
.side-menu-widget-item-checkbox .checkbox-spacer-box {
  margin: 0;
  border: none;
}
